package edu.saldcomp.count;

import io.reactivex.subjects.PublishSubject;
import org.reactivestreams.Publisher;
import org.reactivestreams.Subscriber;
import org.reactivestreams.Subscription;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.function.IntUnaryOperator;

public class Cell implements Publisher<Integer>, Subscriber<Integer> {

    private AtomicInteger value;

    private PublishSubject<Integer> publishSubject;

    private Function<Integer, Integer> function;

    public Cell() {
        this.value = new AtomicInteger();
        this.publishSubject = PublishSubject.create();
        this.function = (i) ->
                this.value.updateAndGet(IntUnaryOperator.identity());

    }

    public Cell(Integer value) {
        this();
        this.value.set(value);
    }

    public Cell(Function<Integer, Integer> function) {
        this();
        this.function = function;
    }

    @Override
    public void subscribe(Subscriber<? super Integer> s) {
        publishSubject.subscribe(s::onNext);
    }

    public void update(Integer i) {
        System.out.println(i);
        value.set(i);
        publishSubject.onNext(i);
    }

    public Integer getValue() {
        return value.get();
    }

    @Override
    public void onSubscribe(Subscription s) {

    }

    @Override
    public void onNext(Integer integer) {
        Integer val = this.function.apply(integer);
        this.value.set(val);
        this.publishSubject.onNext(val);
    }

    @Override
    public void onError(Throwable t) {

    }

    @Override
    public void onComplete() {

    }
}
