package edu.saldcomp.rx;

import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.Scheduler;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.jupiter.api.Test;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.stream.IntStream;

public class SimpleTest {

    private static final Logger LOGGER = LogManager.getLogger(SimpleTest.class.getName());

    @Test
    public void simpleTest() {
        Observable<Integer> data =
                Observable.just(60)
                        .filter(this::isValueBig)
                        .map(this::multiply);

        Disposable subscribe = data.subscribe((value) -> LOGGER.info(value));

    }

    @Test
    public void secondTest() throws InterruptedException {
        Observable.interval(0, 100, TimeUnit.MILLISECONDS)
                .subscribe((value) -> LOGGER.info(value));
        Thread.sleep(1000);
    }

    @Test
    public void threeCellsExample() throws InterruptedException {
        Observable<Long> firstCell = Observable.interval(2000, TimeUnit.MILLISECONDS);
        Observable<Long> secondCell = Observable.interval(5500, TimeUnit.MILLISECONDS);
        Observable.interval(1000, TimeUnit.MILLISECONDS)
                .mergeWith(firstCell).mergeWith(secondCell)
                .buffer(500, TimeUnit.MILLISECONDS)
                .filter((list) -> list.size() == 0)
                .subscribe((value) -> LOGGER.info(value));
        Thread.sleep(5000);
    }

    @Test
    public void threads() throws InterruptedException {

        CountDownLatch latch = new CountDownLatch(2);
        Observable<Long> interval = Observable.interval(100, TimeUnit.MILLISECONDS);

        ExecutorService executorService = Executors.newFixedThreadPool(5);
        Scheduler scheduler = Schedulers.from(executorService);

        interval.observeOn(scheduler)
                .map((value) -> {
                    LOGGER.info("map1");
                    return value;
                })
                .subscribeOn(scheduler)
                .subscribe((value) -> {
                    latch.countDown();
                    LOGGER.info("wartość " + value);
                });

        interval.observeOn(scheduler)
                .map((value) -> {
                    LOGGER.info("map1");
                    return value;
                })
                .subscribeOn(scheduler)
                .subscribe((value) -> {
                    latch.countDown();
                    LOGGER.info("wartość " + value);
                });

        latch.await(10, TimeUnit.SECONDS);


    }


    @Test
    public void createStream() {

        Observable<Object> observable = Observable.create(emitter -> {

            IntStream.range(0, 1_000_000).forEach((value) -> {
                emitter.onNext(value);
            });
            emitter.onComplete();

        });

        observable
                .map((v) -> null)
                .map((v) -> null)
                .map((v) -> null)
                .subscribe((val) -> {
                }, (t) -> {
                    LOGGER.error("", t);
                }, () -> {
                    LOGGER.info("SKONCZYLO");
                });


    }


    private Integer multiply(Integer value) {
        return value * 2;
    }

    private boolean isValueBig(Integer value) {
        return value > 50;
    }


}
